@@ -12,16 +12,17 @@
 #include <getopt.h>
 #include "cpucores.h"
 #include "flist.h"
-#include "screen.h"
+#include "output.h"
 #include "progress.h"
 
 #define MAX_THREADS 50				/* maximum number of threads */
+#define MAX_OUTPUTS 50				/* maximum number of outpus */
 #define NUM_THREADS_FACTOR 0.7		/* defalt number of threads = real number of thread * THREADS_FACTOR */
 #define LLUI long long unsigned int
 #define MAX_FILTER_LEN 1024
 
 /* global variable */
-lnf_mem_t *memp;
+//lnf_mem_t *memp;
 int totalrows = 0;
 pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 flist_t *flist;
@@ -29,6 +30,8 @@ int fileidx = 0;
 unsigned long outputflows = 0;
 progress_t *progressp;
 lnf_filter_t *filterp;
+output_t output[MAX_OUTPUTS];
+int numoutputs;
 char filter[1024];
 
 #define NFDUMPP_FILTER_DEFAULT 0
@@ -54,6 +57,7 @@ int process_file(char *filename, lnf_filter_t *filterp) {
 	int i = 0;
 	int tid;
 	int match;
+	int o;
 
 	tid = (int)pthread_self();
 
@@ -77,11 +81,8 @@ int process_file(char *filename, lnf_filter_t *filterp) {
 
 		/* add to memory heap */
 		if ( match ) {
-			if (memp != NULL) {
-				lnf_mem_write(memp, recp);
-			} else {
-				print_row(recp);
-				outputflows++;
+			for (o = 0; o < numoutputs; o++) {
+				output_write(&output[o], recp);
 			}
 		}
 
@@ -102,38 +103,10 @@ void *process_thread(void *p) {
 	int rows;
 	int tid;
 	char filename[PATH_MAX];
-//	lnf_filter_t *filterp = NULL;
+	int o;
 
 	tid = (int)pthread_self();
 
-/*
-	if (filter[0] != '\0') {
-		switch (filter_type) {
-			case NFDUMPP_FILTER_DEFAULT: 
-					lnf_filter_init(&filterp, filter); 
-					break;
-			case NFDUMPP_FILTER_NFDUMP: 
-					lnf_filter_init_v1(&filterp, filter); 
-					break;
-			case NFDUMPP_FILTER_LIBNF: 
-					lnf_filter_init_v2(&filterp, filter); 
-					break;
-			default:
-					fprintf(stderr, "This should never hapen line: %d\n", __LINE__);
-					exit(1);
-					break;
-		}
-					
-		lnf_filter_init(&filterp, filter);
-
-		if (filterp == NULL) {
-			fprintf(stderr, "[#%x] Can not initialise filter %s\n", tid, filter);
-			return 0;
-		}
-	}
-*/
-
-
 	for (;;) {
 
 		/* get next file */
@@ -158,25 +131,26 @@ void *process_thread(void *p) {
 
 DONE:
 
-	if ( memp != NULL ) {
-		lnf_mem_merge_threads(memp);
+	for (o = 0; o < numoutputs; o++) {
+		output_merge_threads(&output[o]);
 	}
 
-//	lnf_filter_free(filterp);
 	return NULL;
 }
 
 
 int main(int argc, char **argv) {
 
-	lnf_rec_t *recp;
+//	lnf_rec_t *recp;
 	pthread_t th[MAX_THREADS];
 	int i = 0;
 	int numthreads = 1;
 	int sortfield = 0;
 	int sortbits4 = 0;
 	int sortbits6 = 0;
-    char c;
+	int c, o;
+	int numaflags = 0;
+	output_t *outputp;
 //	lnf_filter_t *filterp;
 	
 
@@ -185,18 +159,24 @@ int main(int argc, char **argv) {
 	numthreads = get_cpu_cores() * NUM_THREADS_FACTOR;
 
 	/* initalise one instance of memory heap (share by all threads) */
-	memp = NULL;
+//	memp = NULL;
 	filterp = NULL;
 	progressp = NULL;
-	recp = NULL;
+//	recp = NULL;
 	filter[0] = '\0';
 
-	/* fields in all outpusts */
-	fields_add(LNF_FLD_FIRST);
-	fields_add(LNF_FLD_CALC_DURATION);
+	/* fields in all outpusts  - initalised in all outputs */
+	for (i = 0; i < MAX_OUTPUTS; i++) {
+		output_init(&output[i]);
+		output_set_fmt(&output[i], OFMT_LINE, NULL);			
+		output_field_add(&output[i], LNF_FLD_FIRST);
+		output_field_add(&output[i], LNF_FLD_CALC_DURATION);
+	}
 
+	numoutputs = 1;
+	outputp = &output[0];
 
-	while ((c = getopt_long(argc, argv, "A:O:r:R:T:W;", longopts, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "w:o:A:O:r:R:T:W;", longopts, NULL)) != -1) {
 		switch (c) {
 			case 1: 
 			case 'T': 	/* T option will be removed in future */
@@ -221,11 +201,34 @@ int main(int argc, char **argv) {
 			case 'R':
 				flist_lookup_dir(&flist, optarg);
 				break;
-			case 'A':
-				if (memp == NULL) {
-					lnf_mem_init(&memp);
+			case 'w': 
+				output_set_fmt(outputp, OFMT_BIN_NFDUMP, optarg);
+				break;
+			case 'o': 
+				if (strcmp(optarg, "raw") == 0) {
+					output_set_fmt(outputp, OFMT_RAW, NULL);			
+				} else if (strcmp(optarg, "line") == 0) {
+					output_set_fmt(outputp, OFMT_LINE, NULL);			
+				} else {
+					fprintf(stderr, "Unknown output format \"%s\".\n", optarg);
+					exit(1);
 				}
-				parse_aggreg(memp, optarg);
+				break;
+			case 'A':
+				/* if it is the second A flag we initialise second screen */
+				if (numaflags >= 1) {
+					if (numoutputs >= MAX_OUTPUTS - 1) {
+						fprintf(stderr, "Reached max outputs \"%d\".\n", MAX_OUTPUTS);
+					}
+					outputp = &output[numoutputs];
+					outputp->sortfield = output[numoutputs - 1].sortfield;
+					numoutputs++;
+				}
+
+				if ( ! output_parse_aggreg(outputp, optarg) ) { 
+					exit(1);
+				}
+				numaflags++;
 				break;
 			case 'O':
 				sortfield = lnf_fld_parse(optarg, &sortbits4, &sortbits6);
@@ -233,6 +236,7 @@ int main(int argc, char **argv) {
 					fprintf(stderr, "Unknow or unsupported sort field: %s\n", optarg);
 					exit(1);
 				}
+				output_set_sort(outputp, sortfield, sortbits4, sortbits6);
 				break;
 			case '?':
 				printf("Usage: %s [ -A ] [ -R -r ] [ <filter> ] \n", argv[0]);
@@ -240,6 +244,8 @@ int main(int argc, char **argv) {
 				printf(" -R : Input file or directory  (multiple -r -R options is allowed)\n");
 				printf(" -A : aggregation\n");
 				printf(" -O : sort order\n");
+				printf(" -o : output format: line, raw, nfdump (default: line or nfdump with -w option)\n");
+				printf(" -w : output file\n");
 				printf(" --num-threads = <num> : num threads (default: %.0f%% number of CPU cores, %d on this system)\n", 
 							NUM_THREADS_FACTOR * 100, numthreads);
 				printf(" --filter-type = nfdump|libnf : use original nfdump filter or new libnf implementation \n");
@@ -284,56 +290,17 @@ int main(int argc, char **argv) {
 //		lnf_filter_free(filterp);
 	}
 
+	output_start(&output[0]);
 
 	/* init progress bar */
 	if  (progress_init(&progressp, 0, NULL) != NULL) {
 		progress_steps(progressp,  flist_count(&flist));
 	}
 
-	/* aggregated or not aggregated records */
-	if (memp == NULL) {
-		/* not aggregated, but sorted */
-		if (sortfield > 0) {
-			lnf_mem_init(&memp);
-			/* switch memp into list mode */
-			lnf_mem_setopt(memp, LNF_OPT_LISTMODE, NULL, 0);
-			lnf_mem_fastaggr(memp, LNF_FAST_AGGR_BASIC);
-			lnf_mem_fadd(memp, LNF_FLD_PROT, LNF_AGGR_KEY, 0, 0);
-			lnf_mem_fadd(memp, LNF_FLD_SRCADDR, LNF_AGGR_KEY, 24, 128);
-			lnf_mem_fadd(memp, LNF_FLD_SRCPORT, LNF_AGGR_KEY, 0, 0);
-			lnf_mem_fadd(memp, LNF_FLD_DSTADDR, LNF_AGGR_KEY, 24, 128);
-			lnf_mem_fadd(memp, LNF_FLD_DSTPORT, LNF_AGGR_KEY, 0, 0);
-		}
-    	fields_add(LNF_FLD_PROT);
-    	fields_add(LNF_FLD_SRCADDR);
-    	fields_add(LNF_FLD_SRCPORT);
-    	fields_add(LNF_FLD_DSTADDR);
-    	fields_add(LNF_FLD_DSTPORT);
-	}
-
-	/* default fields on the ond of the list */
-    fields_add(LNF_FLD_DPKTS);
-    fields_add(LNF_FLD_DOCTETS);
-    fields_add(LNF_FLD_CALC_BPS);
-    fields_add(LNF_FLD_CALC_BPP);
-    fields_add(LNF_FLD_AGGR_FLOWS);
-
-	/* set sort firld */
-	if (sortfield > 0) {
-		int defaultaggr = 0;
-		int defaultsort = 0;
-		lnf_fld_info(sortfield, LNF_FLD_INFO_AGGR, &defaultaggr, sizeof(int));
-		lnf_fld_info(sortfield, LNF_FLD_INFO_SORT, &defaultsort, sizeof(int));
-		lnf_mem_fadd(memp, sortfield, defaultaggr|defaultsort, sortbits4, sortbits6);
-	}
-
-
-	print_header();
-
 	/*  prepare and run threads */
 	pthread_mutex_init(&mutex, NULL);
 
-	for ( i = 0 ; i < numthreads ; i++ ) {
+	for (i = 0 ; i < numthreads ; i++ ) {
 		if ( pthread_create(&th[i], NULL, process_thread, NULL) < 0) {
 			fprintf(stderr, "Can not create thread for %d\n", i);
 			break;
@@ -341,34 +308,34 @@ int main(int argc, char **argv) {
 	}
 
 	/* wait for threads */
-	for ( i = 0; i < numthreads; i++ ) {
+	for (i = 0; i < numthreads; i++ ) {
 		if( pthread_join(th[i], NULL) ) {
 			fprintf(stderr, "Error joining thread\n");
 			break;
 		}
 	}
 
-	/* print the records out */
-	if (memp != NULL) {
-		i = 0;
-		lnf_rec_init(&recp);
-		while (lnf_mem_read(memp, recp) != LNF_EOF) {
-			i++;
-			print_row(recp);
-			outputflows++;
+	for (o = 0; o < numoutputs; o++) {
+		if (o > 0 ) { 
+			output_start(&output[o]);
 		}
+
+		/* print the records out */
+		output_output_rows(&output[o]);
+
+
+		/* finish output */
+		output_finish(&output[o]);
+
+		/* header */
+		printf("Total input flows %d, output flows: %lu\n\n", totalrows, output[o].outputflows);
+
 	}
 
-	/* header */
-	printf("Total input flows %d, output flows: %lu\n", totalrows, outputflows);
-
-	lnf_mem_free(memp);
-	lnf_rec_free(recp);
 	if (filterp != NULL) {
 		lnf_filter_free(filterp);
 	}
 
-
 }
 
 
