@@ -222,6 +222,8 @@ typedef void lnf_mem_cursor_t;
 #define LNF_APPEND			0x10
 #define LNF_ANON			0x02
 #define LNF_COMP			0x04
+#define LNF_COMP_LZO		0x04
+#define LNF_COMP_BZ2		0x20
 #define LNF_WEAKERR			0x08
 
 #define LNF_ERR_UNKBLOCK	-0x0001	/* weak error: unknown block type */
@@ -250,6 +252,8 @@ typedef void lnf_mem_cursor_t;
 #define LNF_INFO_FILE_VERSION	0x06	/* nfdump file version  - int*/
 #define LNF_INFO_BLOCKS			0x08	/* number of block in file - unit64_t */
 #define LNF_INFO_COMPRESSED		0x0A	/* is file compressed - int */
+#define LNF_INFO_LZO_COMPRESSED	0x0B	/* is file LZO compressed - int */
+#define LNF_INFO_BZ2_COMPRESSED	0x0D	/* is file BZ2 compressed - int */
 #define LNF_INFO_ANONYMIZED		0x0C	/* is file anonymized - int */
 #define LNF_INFO_CATALOG		0x0E	/* file have catalog - int */
 #define LNF_INFO_IDENT			0x10	/* string ident - char* */
@@ -309,7 +313,9 @@ structure (see record operations).
 	LNF_APPEND - open file for reading in append mode \n
 	LNF_WRITE - open file for for writing  \n
 	LNF_ANON - set anon flag on the file (only for write mode) \n
-	LNF_COMP - set the output file to be compressed \n
+	LNF_COMP - set the output file to be compressed (using LZO)\n
+	LNF_COMP_LZO - set the output file to be compressed (using LZO)\n
+	LNF_COMP_BZ2 - set the output file to be compressed (using BZ2)\n
 	LNF_WEAKERR - when reading reports also weak errors (unknown block, ...) \n
 \param *ident 		file ident for newly created files, can be set to NULL
 \return 			LNF_OK, LNF_ERR_NOMEM
@@ -598,11 +604,12 @@ PORT    PKTS BYTES
 int lnf_mem_setopt(lnf_mem_t *lnf_mem, int opt, void *data, size_t size);
 
 /* flags for lnf_mem_addf */
-#define LNF_AGGR_KEY	0x0000	/* the key item */
+#define LNF_AGGR_AUTO	0x0000	/* autodetect aggregation key */
 #define LNF_AGGR_MIN	0x0001	/* min value - for LNF_FLD_FIRST */
 #define LNF_AGGR_MAX	0x0002	/* max value - for LNF_FLD_LAST */
 #define LNF_AGGR_SUM	0x0003	/* summary of values - for all counters */
 #define LNF_AGGR_OR		0x0004	/* OR operation - for LNF_TCP_FLAGS */
+#define LNF_AGGR_KEY	0x0008	/* key field */
 #define LNF_AGGR_FLAGS	0x000F
 
 #define LNF_SORT_NONE	0x0000	/* do not sort by this field */
@@ -822,7 +829,7 @@ int lnf_fld_type(int field);
 #define LNF_INFO_BUFSIZE 4096		/* maximum buffer size for data lnf_*_fields */
 /* return LNF_OK or LNF_ERR_UNKFLD or LNF_ERR_OTHER */
 int lnf_fld_info(int field, int info, void *data, size_t size);
-int lnf_fld_parse(char *str, int *numbits, int *numbits6);
+int lnf_fld_parse(const char *str, int *numbits, int *numbits6);
 
 
 #ifndef IN6_IS_ADDR_V4COMPAT
