@@ -15,7 +15,7 @@ use threads;
 
 our @ISA = qw(Exporter);
 
-our $VERSION = '1.17';
+our $VERSION = '1.18';
 
 # XXX
 # Items to export into callers namespace by default. Note: do not export
@@ -290,6 +290,9 @@ sub set_fields {
 		}
 	}
 
+	$self->{NAME} = $self->{fields_txt};
+	$self->{NUM_OF_FIELDS} = scalar @{$self->{fields_txt}};
+
 	$self->{opts}->{Fields} = $self->{fields_txt};
 
 	return Net::NfDump::libnf_set_fields($self->{handle}, $self->{fields_num});
@@ -522,6 +525,13 @@ sub DESTROY {
 
 This method has to be applied before any of the C<fetchrow_*> methods is used. Any option described before can be used as a parameter of the method. 
 
+After executing query command it possible to access $flow->{NUM_OF_FIELDS} and $flow->{NAME} variable to get returnd 
+number of fields and field names. Here is an exmaple of code to acces field names: 
+
+    foreach $colno (0..$flow->{NUM_OF_FIELDS}-1) {
+        print $flow->{NAME}->[$colno]."\t";
+    }
+
 =cut 
 
 # Query method can be used in two ways. If the string argument is the 
